#ifndef _FLTE
#define _FLTE

#include <stdio.h>

#include "slp.h"
#include "l1data.h"

#define FLTE_MAX_TRIGGERS 50
#define FLTE_MAX_TRACKS 100

SLP_RETURN FLTE_initialize();
void FLTE_zero();
SLP_RETURN FLTE_handle(int isRPS, int msg_class, void *header, void *data);
SLP_RETURN FLTE_trigger(int event);
void FLTE_free(int event);
void FLTE_printEvent(FILE *out, int event); 

struct FLT_TRACK {  /* This is identical to L2ROI, for the moment */
  PARTTYPE trackID;      /* pretrigger track ID */
  float energy;          /* total energy reconstructed */
  float e_err;           /* error on energy, or 0. if unavailable */
  float pxpz;            /* Px/Pz */
  int charge;            /* charge */
  
  float x1;              /* The RoI definition: */
  float x1_err;
  float y1;
  float y1_err;
  float z1;
  float x2;
  float x2_err;
  float y2;
  float y2_err;
  float z2;
};
typedef struct FLT_TRACK FLT_TRACK;

/* For use by SLT */

const L2FR_L1Data  *FLTE_getHeader(int event);
const L2FR_L1Trig  *FLTE_getTriggers(int event);
const FLT_TRACK    *FLTE_getTracks(int event);

/* For getting and setting trigger parameters */

void  FLTE_setPtThreshold(float thresh);
float FLTE_getPtThreshold(void);
void  FLTE_setECMultiplier(float mult);
float FLTE_getECMultiplier(void);
void  FLTE_setMassCut(float mcut);
float FLTE_getMassCut(void);
void  FLTE_setTargetStretch(float stretch); /*** in cm. Target box is widened by this amount ***/
float FLTE_getTargetStretch(void);
void  FLTE_printStats(FILE *f);

#endif
