/***
 *
 *	L2defs.h
 *	
 *	define MOD and some associated items.
 *	
 ****/

#ifndef _L2DEFS
#define _L2DEFS

#include "l2mess.h"
#include "l2net.h"
#include "btable.h"

#ifndef TRUE
#undef FALSE
typedef enum {FALSE, TRUE} BOOLEAN;
#else
typedef int BOOLEAN;
#endif

typedef enum { L2_UNASSIGNED, L2_ASSIGNED, L2_READ_ONLY ,L2_FED} PORTFLAG;

typedef struct {
	char *key;
	int   nPorts;
} ModType;

struct Port{
	PORTFLAG ioFlag;
	int *callFlag;
	Message *in;
	Message **out;
/*    START_CIL */
	int isFREE;
	float busyTime;
/*    STOP_CIL  */
};
typedef struct Port Port;

/***

   the Proc structure is a table element which associates 
   an ascii keyword with a pointer to each function that is
   intended to run on (or model) a DSP process. It is used
   to decipher the configuration file.

   The table is configuration dependent and kept in ptable.c

***/

typedef struct {
	char *key;
	void (*DoCommand)(int isRPS, int msg_type, void *, void *);
	char *imageFile;
	int sharcAddr; /* Mod address on sharc. filled from config file*/
} Proc;

/***
 *
 * This is hopefully temporary. It is intended to facilitate getting 
 * addresses into the sharc. Once host communications is better sorted
 * out it can in principle disappear.
 *
 ***/

struct L2ADDRLIST {
  int nAddrs;
  RPS_addr theAddrs[1];
};
typedef struct L2ADDRLIST L2ADDRLIST;



enum VARS {           /* the list of VAR pointers */
  DAQ,
  EV_CONTROL,
  EV_BUILDER,
  SLB,

  GRR, 
  EV_BUFF,           /* For event storage on SLT processor nodes */
  PROC_GLOBAL,       /* General for SLT processor nodes */

  L2_SLICER,         /* SLT algorithm sections */
  L2_REFITX,
  L2_REFITY,
  L2_MAGNETTRACK,
  L2_VERTEXTRACK,
  L2_TARGET,

  L2_XBACKGEO,       /* Detector sections */
  L2_STEREOBACKGEO,
  L2_MAGNETGEO,
  L2_VERTEXGEO,
  L2_TARGETGEO,

  L2_XBACKSLB,       /* SLB maps */
  L2_STEREOBACKSLB,
  L2_MAGNETSLB,
  L2_VERTEXSLB,
  L2_TARGETSLB,

  L3_QUESTION_MARK,
  L4_QUESTION_MARK,

  MAX_VARS           /* this should always be last */
};


/***
 *
 *  typedef MOD:
 *       
 ***/

struct MOD{

        int magic;            /* magic number. to check for correct uploading */
	char *key;	      /* keyword defined by a "module" directive */
	int copy;	      /* copy number */
        char *name;           /* == key_copy. redundant but heavily used */
	ModType *m;	      /* the module type given in the "module" directive */
	char *procKey;        /* to identify process table entry */
                              /* the command processor for this module (invisible) */
	void (*DoCommand)(int isRPS, int msg_class, void *header, void *data);

	void *var[MAX_VARS];  /* for local "static" variables */

	int routerID;         /* router id of my router w.r.t routing tables == RPS address */
	int boardPos;         /* board position of this module */
	int nMessages;        /* the number of messages received */

/* The following is just used by the simulation. It should not be referenced in the procs. */

	BOARD *myBoard;       /* pointer to my board structure (invisible) */

	float userTime;       /* time since proc invocation. Set with INCREMENT_UTIME (invisible)*/
	float muserTime; 
	float ruserTime;
	float busyTime;       /* cpu usage since beginning of simulation - maintained by Scheduler */

	int callMe[2];        /* if non-zero, call L2net_Comm on this cycle */
	int isFree;	      /* non-zero if Mod is available for scheduling (invisible) */
	
        int nNodes;           /* number of nodes in routing table (invisible) */
        L2NODE *routes;       /* routing table (invisible) */
        int nAddrLists;       /* the number of address lists (invisible) needed for automatic param generation */
        L2ADDRLIST *addrList; /* the address lists. (invisible) needed for parameter generation */

	Port inq;             /* input port for L2comm messages (invisible) */
	Port *links;          /* array of ports for link communications via L2net (invisible) */
	int nGlobals;         /* number of global bus "ports" */
	Port *gBus;           /* array of ports for communication over global bus by L2net (invisible) */
};

typedef struct MOD MOD;


#define INCREMENT_UTIME(delta) current->userTime += (delta); current->muserTime += (delta)

#define MyAddress current->routerID

#ifndef _L2_MTABLE_DOT_C
extern MOD *current;
#else
MOD *current;
#endif

/***
 *
 * global variable L2simu_mess is used for returning monitoring messages
 * to the simulation code which, in the real world would go via the
 * switch.
 *
 ***/

#ifndef _L2_MTABLE_DOT_C
extern RPS_msg *L2simu_mess;
#else
RPS_msg *L2simu_mess = 0;
#endif

#endif
