#ifndef _L2GROUP
#define _L2GROUP

#include "l2defs.h"

/***

   These enumeration values serve two purposes, to specify an
   iterator type and except for GTYPE_GENERIC, as bit masks.
   The bits are numbered starting from 1. If a new type is added,
   it should be a power of two.

   It is assumed that the group table is set up using directives in
   the configuration file only. If an attempt is made to alter the
   tables by some other means, I ain't responsible for the core dump.

 ***/

typedef enum {GTYPE_GENERIC = 0, GTYPE_BROADCAST = 1, GTYPE_EVA = 2} GTYPE;

struct L2GROUP;
typedef struct L2GROUP L2GROUP;

/* l2tgroup (the table of groups */

void   l2tgroup_Print(void);
void   l2tgroup_Destroy(void);

const L2GROUP *l2tgroup_Lookup(const char *groupName);
const L2GROUP *l2tgroup_IterateInit(void);
const L2GROUP *l2tgroup_IterateNext(void);

int l2tgroup_NumGroups(GTYPE type);

/* l2group (the groups) */


const L2GROUP *l2group_Construct(int argc, const char **argv);

/***
   * I'm experimenting with style here. Note that the definition of L2GROUP
   * is hidden inside of l2group.c. Hence the need for all of these little
   * access functions. Theoretically it makes maintenance easier since the
   * structure of L2GROUP is completely hidden. But it ain't without drawbacks.
   *
   ***/

int         l2group_AssignAddress( const L2GROUP *group ,RPS_addr addr);
GTYPE       l2group_GetType(const L2GROUP *group);
const char *l2group_GetName(const L2GROUP *group);
RPS_addr    l2group_GetAddressG(const L2GROUP *group);
RPS_addr    l2group_GetAddress( const char *name );
int         l2group_IsMember(const L2GROUP *, const MOD *);

void        l2group_Print( const L2GROUP *group );

const MOD  *l2group_IterateInit( const L2GROUP *group);
const MOD  *l2group_IterateNext( const L2GROUP *group);

#endif

