#ifndef _L2IO
#define _L2IO

#include "event.h"      /* online event header */
#include "component.h"  /* online component definitions */
#include "l2event.h"

/*   compid in the FED header is COMP_ID_SLT + appropriate L2SUBCOMP_ID    */

enum L2COMP_ID {   /* If you add anything to this list, put it at the end. */
  L2COMP_EVENT,
  L2COMP_EVENT_AUX,
  L2COMP_TRIG,
  L2COMP_TRACK,
  L2COMP_ECAL,
  L2COMP_SLICER,
  L2COMP_SLICER_HIT,
  L2COMP_REFIT,
  L2COMP_REFIT_HIT,
  L2COMP_MAGNET,
  L2COMP_MAGNET_HIT,
  L2COMP_SILI,
  L2COMP_SILI_HIT,
  L2COMP_VERTEX
};

enum L2IO_RETURNS { /* return codes */
  L2IO_SUCCESS,
  L2IO_DONE,
  L2IO_NOT_FOUND,
  L2IO_TOO_BIG
};

#define L2IO_MAGIC 0x12345678
#define L2IO_EVENT_VERS 0

struct L2IO_EVENT {
  unsigned int magic;        /* to check the endianess (is it a word? maybe endianitude)   */
  unsigned int version;      /* version number of this bank                                */
  unsigned int eventType;    /* to be defined. e.g. FLT, ECAL_PRE, pulser...               */
  unsigned int procNode;     /* processor number on which SLT code was run                 */
  unsigned int nTriggers;    /* number of FLT triggers                                     */
  unsigned int nTracks;      /* number of SLT tracks = #trigger-associated FLT tracks      */
  L2ALGOSTAT algo;           /* last algorithm section which executed                      */
  unsigned int acceptMask;   /* event status (accept/reject) for each FLT trigger          */
  unsigned int trigMask;     /* which trigger types are satisfied by any trigger           */
  unsigned int l2compMask;   /* mask of L2 sub components in event record                  */
  long time;                 /* As returned by time(), filled during event build           */
                             /* Note: I use long instead of time_t, assuming linux writes  */
};
typedef struct L2IO_EVENT L2IO_EVENT;

/*** 

     Auxilliary bank for event-related stuff which is needed if more than just the
     top level event block is output. The idea is to keep only summary information
     in the top block.

***/

struct L2IO_EVENT_AUX {
  L2GEO_VERSION geo;                     /* Geometry set identifiers                           */
  int nFLTTracks;                        /* # of FLT tracks in this event                      */
  int SLTTrack[FLTE_MAX_TRACKS+1];     /* SLT trk num from FLT trk num or -1 if non-existent */
};
typedef struct L2IO_EVENT_AUX L2IO_EVENT_AUX;
  
                                                              /* Build full L2 output in buff */
int L2IO_BuildAll(int event, int l2compMask, char *buff, int *len, int maxLen); 

int L2IO_Init(int event, int l2compMask);                     /* Initializes a building sequence */
int L2IO_BuildNext(char *buff, int *len, int maxlen);/* also add FED header/trailer, swap */


#endif

