#ifndef _PLTABLE
#define _PLTABLE

#include <stdio.h>

typedef struct {
   int l2npl;     /* absolute number of plane               */ 
   int l2nsl;     /* absolute number of superlayer          */ 
   int cmp;       /* detector component 1 (SIL),2 (WCH),... */
   int lay;       /* layer number within superlayer         */
   int sec;       /* sector number                          */
   int mod;       /* module number for outer tracker        */
   int shp;       /* detector shape                         */
   float xmin;    /* x of the left  edge of the detector    */
   float xmax;    /* x of the right edge of the detector    */
   float ymin;    /* y of the left  edge of the detector    */ 
   float ymax;    /* y of the right edge of the detector    */
   float zmin;    /* z of the front side                    */
   float zmax;    /* z of the rear side                     */
   float sxmn;    /* x of the left  edge of the sens. area  */
   float sxmx;    /* x of the right edge of the sens. area  */
   float symn;    /* y of the left  edge of the sens. area  */
   float symx;    /* y of the right edge of the sens. area  */
   float cos;     /* cosine of the stereo angle             */
   float sin;     /* sine   of the stereo angle             */
   float rot[9];  /* 3 by 3 rotation matrix                 */
   float acc;     /* measurement accuracy                   */
   float eff;     /* hit efficiency                         */
   float dod;     /* max drift distance or double hit reso  */
   float pit;     /* pitch                                  */
   float ms;      /* multiple scattering angle for 1 GeV    */
   float xfw[4];  /* x of first wire in sublayer            */
   float zfw[4];  /* z of first wire in sublayer            */
   int nwi[4];    /* number of wires in sublayer            */
   int nsl;       /* number of sublayers in segment         */
    int alphaGarbage; /*get byte alignment right for alpha  */
} PLANE;


PLANE *PlLookup(int);       /* for use by Geo only */
void PlInsert( PLANE *);    /* for use by Geo only */
void Pl_PrintTable(FILE *);
void Pl_PrintPlane(FILE *,int);
void Pl_ReadPlane(FILE *, PLANE **);
void Pl_PrintPlaneSpy(FILE *,int);

void L2_excludePlane(const char sl_name[4], int sector, int layer);
int  Pl_isExcluded(const PLANE *pl);
void Pl_PrintExcludeTable(void);

PLANE *Pl_IterateInit(const char *sl_name); /* name == NULL ==> all planes */
PLANE *Pl_IterateNext();

#endif
