/*;*/
/**  rps inline or static function definitions.

rps message system definitions utility functions
are provided - No ASM support - Should be included
from rps.h; else the macro "inkey" has to be defined
to be ether "inline static" or "static" or "" depending
on the usage.

Be aware an implementation of RPS_my_address() is missing
from this file and should be provided elsewhere. If it is an
inline it should be defined such before(!) this file is included.
It is implemented for the SHARC in "rps_sfun.h"

@name rps_fun.h
; */
/*@{*/

#ifndef __RPS_FUN__
#define __RPS_FUN__



#ifdef __DEF_GLOBAL__
#ifdef inkey
#define backkey inkey
#endif
#undef inkey
#define inkey
#endif


/** Fill message header.
All fields of the message header are filled according to
the parameters. The source field is filled withe the
local address as specified to the RPS_init call.


@param mes  pointer to message
@param to   destination address
@param len  length of to be sent message (message content only)
@param id   message id
@param type message type

@name RPS_build
 */
inkey void RPS_build(RPS_msg *mes,RPS_addr to,int len,int id,RPS_type type){
  mes->hdr.source = RPS_my_address();
  mes->hdr.dest   = to;
  mes->hdr.length = len;
  mes->hdr.ident  = id;
  mes->hdr.type   = type;
  mes->hdr.after  = (RPS_msg *)0;
}



/** Fill origin field in header with specific subaddress.
The source field in the message header is filled with the SHARCs
address and the specified subaddress. The subaddress has to obey
the RPS_adr format - ie it has to go to the byte 2. 
This call allows to specify one specific return address - e.g. for
a request.

@param mes pointer to message
@param src source sub-address to be filled into the origin field

@name RPS_source
 */
inkey void RPS_source(RPS_msg *mes,RPS_sub_addr sub){
  mes->hdr.source = RPS_put_sub_addr(RPS_my_address(),sub);
}

/** Get length for RPS messages.
Return length used for RPS messages in int*4. This length
excludes the space required for the RPS header is therefore the
usable message space.

@return length of rps messages.

@name RPS_length
 */
inkey int RPS_length(void){
  return RPS_MSG_SIZE;
}

/** Point to RPS message contents.

@param m pointer to RPS message

@return pointer to contents of RPS message.

@name RPS_content
 */
inkey int *RPS_content(RPS_msg *m){
  return (int *)(m+1);
}


#ifdef __UNIX__
inkey RPS_msg *RPS_alloc(void);

inkey RPS_msg *RPS_dup(RPS_msg *mes){
  RPS_msg *n;
  int len,*f,*t;

  n = RPS_alloc();
  n->hdr = mes->hdr;
  len = (n->hdr.length+3)<<2;
  t = RPS_content(n); f = RPS_content(mes);
  while(len--)
    *(t++)=*(f++);
}
#else
inkey RPS_msg *RPS_dup(RPS_msg *mes){
  RPS_msg *new;
  asm volatile ("call RPS_dup (DB);i3=%1;nop;%0=i3;":"=r" (new) :"r" (mes):\
      "r0","r1","r12","i1","i11","i5","i3");
  return new;
}
#endif


/** Send message and send another one when it is sent out.

The header field mes->hdr.after is filled with sec. Then RPS_send is
called for mes. This causes RPS to send sec out (according to its
header information) after the DMA for mes has been concluded.

BUGs and SHORTCOMMINGs:
Be aware! This does not work when the first message is the local
process since it never gets handed to the DMA handlers. The second
message is only sent after completion of a DMA!

@param mes  message to be send
@param sec  message to be send when mes has left the SHARC

@return
RPS_ok normal return

@name RPS_send2
 */
inkey int RPS_send2(RPS_msg *mes,RPS_msg *sec){
  mes->hdr.after = sec;
  RPS_send(mes);
}




#ifdef backkey
#undef inkey
#define inkey backkey
#undef backkey
#endif


#endif



/*@}*/
