#ifndef _SLP_EVENTS
#define _SLP_EVENTS

#include "slp.h"
#include "slp_msg.h"

typedef enum SLP_SLOT_STATUS {
  SLP_SLOT_EMPTY,       /* If no event is in slot (initial value) */
  SLP_SLOT_AVAILABLE,   /* If an event is in slot which can be overwritten (freeEvent sets to this) */
  SLP_SLOT_LOCKED,      /* If the slot is taken (set to this by insert) */
  SLP_SLOT_NON_EXISTANT /* A possible return for SLP_getSlotStatus */
} SLP_SLOT_STATUS;

#define SLPE_ERROR -1

/***
   *
   * A container, for TSLP_EVENTs. Events can be inserted and freed. On insertion, the event
   * is assigned a "local event number" which can be used for retrieving event info and must
   * be used when freeing the slot. A complication is that, in simulation mode, it is necessary
   * to have access to the event even after it is freed, since it must be freed before returning
   * from l2evnt_ but users will want access to event info. Hence the SLP_SLOT_AVAILABLE status.
   * In this state, the slot can be reallocated by l2simu (but won't be before the next call to
   * l2evnt_. In simulation mode, the FCSevt number is the arte event number.
   *
   ***/

void SLP_events_init();               /* initialize the event list                                      */
SLP_RETURN SLP_insert(TSLP_EVENT *event);/* insert event                                                */
void SLP_freeSlot(int evt);           /* release event slot (mark as available)                         */

int SLP_getLocalNumber(int FCSevt);   /* Return the local event number, given FCS number (or SLPE_ERROR)*/
int SLP_getFCSNumber(int evt);        /* return the FCS event number (or SLPE_ERROR)                    */
const TSLP_EVENT *SLP_getEvent(int evt); /* return a const pointer to the event structure( or 0)        */

int SLP_getMaxSlots( void );          /* Return the maximum number of events which can be stored        */
SLP_SLOT_STATUS SLP_getSlotStatus(int evt); /* return status of slot occupied by evt                    */
int SLP_getNLocked( void );           /* Return the number of locked slots                              */
int SLP_getNAvailable( void );        /* Return the number of available + empty slots                   */

#endif
