/*
**++
**  FACILITY:  HERA-B_STATE_MACHINE_CONTROL_PACKAGE
**
**  MODULE DESCRIPTION:
**
**      State machine control package.
**
**  AUTHORS:
**
**      V.Rybnikov
**
**  CREATION DATE: 07/09/1997 
**
**  DESIGN ISSUES:
**
**  The header file for the HERA-B State Machine Control Package.
**
**  MODIFICATION HISTORY:
**
**  07/09/97    VR  first version
**  20/10/97    VR  RPS package for SHARCs added 
**  24/11/97    VR  SMC_CONFIG type message added, SMC_CONFIG_MSG format 
**                  definition
**  05/12/97    VR  input parameter switches and defaults definitions
**  16/12/97    VR  SMC_CHKCFG,  SMC_CONFIGL, SMC_STATEALLRQ  added
**  18/12/97    VR  smc_add_proc_in_group(), smc_rem_proc_from_group() added
**  19/12/97    VR  rccls member has been added to PGT
**  02/01/98    VR  SMC_HELLO, SMC_ACK added, SMC_BYEBYE has been changed
**  06/01/98    VR  SMC_CLASS message user's handler can be used
**  07/01/98    VR  privilege mask added to CPT
**  20/01/98    VR  smc_set_pgt, smc_set_group_num added, user's PGT
**                  creation implemented              
**  28/01/98    VR  environment variables definitions
**  20/02/98    VR  all states are stored in SMC_STS structures
**                  storing the error status for all processes
**  23/02/98    VR  functions added: 
**                  smc_set_err_status(), smc_get_err_status(),
**                  smc_errstat_itoa() for error status manipulations.
**  25/02/98    VR  functions added: 
**                  smc_startup(), smc_halt() - must be used if smc_loop()
**                                              is not used;
**                  smc_start_thread() - requset for starting user's 
**                                       main_loop as a thread (Lynx).
**                  smc_handler_addr() - obtaining the SMC handler address
**  06/03/98    VR  state transition table id is added to the daughter list  
**            
**--
*/
 
#ifndef __HERA_B_SMC__ 
#define  __HERA_B_SMC__
  
#include "grr.h"


  /*
  **       SMC system error CODES
  */


typedef enum SMC_CODES {
    SMC_OK = 0,        /* normal succesful return                           */
    SMC_ERROR,         /* error return                                      */
    SMC_NOFILE,        /* file doesn't exist                                */
    SMC_BAD_FORMAT,    /* the format of the file is wrong                   */
    SMC_BAD_DATA,      /* data in the file is invalid                       */
    SMC_BADPRM,        /* bad parameter for a routine                       */
    SMC_FAILURE,       /* the additional code 
			  for the user's state transition routines          */
    SMC_ROUTE,         /* no direct transition allowed 
			  from one state to another                         */
    SMC_NOCMD,         /* no state found in STT                             */
    SMC_NOMORE,        /* no more resource for the operation                */
    SMC_NOPROC,        /* a process missed in the group with AND logic      */
    SMC_TRANS          /* a process in transition */
} SMC_CODES;

#define SMC_CLASS  26

SMC_CODES smc_get_cur_state();
void      smc_set_cur_state(SMC_CODES newState);

#endif
